import { ConstructorOf } from '@Types/ConstructorOf';
import TypeBase from '@Types/TypeBase';
import ObjectBoolean from '@Types/ObjectBoolean';
export default abstract class OperatorBase<LeftType extends TypeBase, RightType extends TypeBase> {
    /**
     * @protected
     * @type {TypeBase}
     * @memberof OperatorBase
     */
    protected check: ConstructorOf<TypeBase>;
    /**
     * @protected
     * @type {LeftType}
     * @memberof Operator
     */
    protected leftType: LeftType;
    /**
     * @protected
     * @type {RightType}
     * @memberof Operator
     */
    protected rightType: RightType;
    /**
     * Creates an instance of OperatorBase.
     * @param {ConstructorOf<TypeBase>} check
     * @memberof OperatorBase
     */
    constructor(check: ConstructorOf<TypeBase>);
    /**
     * @abstract
     * @returns {TypeBase}
     * @memberof Operator
     */
    abstract apply(): TypeBase;
    /**
     * @abstract
     * @returns {TypeBase}
     * @memberof Operator
     */
    abstract name(): string;
    /**
     * @protected
     * @returns {(void | never)}
     * @memberof OperatorBase
     */
    protected typeCheckLeft(): ObjectBoolean | never;
    /**
     * @protected
     * @returns {(void | never)}
     * @memberof OperatorBase
     */
    protected typeCheckRight(): ObjectBoolean | never;
    /**
     * @param {LeftType} Type
     * @memberof Operator
     */
    left(Type: LeftType): this;
    /**
     * @param {RightType} Type
     * @memberof Operator
     */
    right(Type: RightType): this;
}
