import { ConstructorOf } from '@Types/ConstructorOf';
import TypeBase from '@Types/TypeBase';
import ObjectFunction from '@Types/ObjectFunction';
import ObjectVoid from '@Types/ObjectVoid';
import ObjectInt from '@Types/ObjectInt';
import ObjectNull from '@Types/ObjectNull';
/**
 * @export
 * @class ObjectArray
 * @extends {TypeBase<Z[]>}
 * @template Z
 */
export default class ObjectArray<Z extends TypeBase> extends TypeBase<Z[]> {
    [index: number]: TypeBase;
    /**
     * @private
     * @type {ConstructorOf<Z>}
     * @memberof ObjectArray
     */
    private matcher;
    /**
     * @private
     * @type {number}
     * @memberof ObjectArray
     */
    private size;
    /**
     * Creates an instance of ObjectArray.
     * @param {ConstructorOf<Z>} type
     * @param {Z[]} [arg=[]]
     * @memberof ObjectArray
     */
    constructor(type: ConstructorOf<Z>, arg?: Z[]);
    /**
     * @template T
     * @param {T} _type
     * @returns {ObjectArray<Z>}
     * @memberof ObjectArray
     */
    cast<T extends TypeBase<unknown>>(_type: T): ObjectArray<Z>;
    /**
     * @returns {ObjectVoid}
     * @memberof ObjectArray
     */
    empty(): ObjectVoid;
    /**
     * @template T
     * @param {ObjectFunction<T>} fn
     * @returns {ObjectVoid}
     * @memberof ObjectArray
     */
    forEach<T extends (el: Z, index: ObjectInt, array: ObjectArray<Z>) => ObjectVoid>(fn: ObjectFunction<T>): ObjectVoid;
    /**
     *
     * @param {ObjectInt} position
     * @returns {(Z | ObjectNull)}
     * @memberof ObjectArray
     */
    item(position: ObjectInt): Z | ObjectNull;
    /**
     * @returns {ObjectInt}
     * @memberof ObjectArray
     */
    length(): ObjectInt;
    /**
     * @returns {string}
     * @memberof ObjectArray
     */
    name(): string;
    /**
     * @returns {boolean}
     * @memberof ObjectArray
     */
    nullable(): boolean;
    /**
     * @returns {Z}
     * @memberof ObjectArray
     */
    pop(): Z;
    /**
     * @param {Z} el
     * @returns {ObjectInt}
     * @memberof ObjectArray
     */
    push(el: Z): ObjectInt;
    /**
     * @param {ObjectArray<Z>} [value]
     * @returns {typeof value extends undefined ? Z[] : []}
     * @memberof ObjectArray
     */
    value(value?: ObjectArray<Z>): typeof value extends undefined ? Z[] : [];
}
