"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var TypeBase_1 = __importDefault(require("@Types/TypeBase"));
var ObjectVoid_1 = __importDefault(require("@Types/ObjectVoid"));
var ObjectInt_1 = __importDefault(require("@Types/ObjectInt"));
var ObjectNull_1 = __importDefault(require("@Types/ObjectNull"));
/**
 * @export
 * @class ObjectArray
 * @extends {TypeBase<Z[]>}
 * @template Z
 */
var ObjectArray = /** @class */ (function (_super) {
    __extends(ObjectArray, _super);
    /**
     * Creates an instance of ObjectArray.
     * @param {ConstructorOf<Z>} type
     * @param {Z[]} [arg=[]]
     * @memberof ObjectArray
     */
    function ObjectArray(type, arg) {
        if (arg === void 0) { arg = []; }
        var _this = _super.call(this, arg) || this;
        /**
         * @private
         * @type {number}
         * @memberof ObjectArray
         */
        _this.size = 0;
        _this.matcher = type;
        _this.size = arg.length;
        for (var i = 0; i < arg.length; i++) {
            _this[i] = arg[i];
        }
        return _this;
    }
    /**
     * @template T
     * @param {T} _type
     * @returns {ObjectArray<Z>}
     * @memberof ObjectArray
     */
    ObjectArray.prototype.cast = function (_type) {
        throw new Error('Method not implemented.');
    };
    /**
     * @returns {ObjectVoid}
     * @memberof ObjectArray
     */
    ObjectArray.prototype.empty = function () {
        for (var i = 0; i < this.length().value(); i++) {
            delete this[i];
        }
        this.size = 0;
        return new ObjectVoid_1.default();
    };
    /**
     * @template T
     * @param {ObjectFunction<T>} fn
     * @returns {ObjectVoid}
     * @memberof ObjectArray
     */
    ObjectArray.prototype.forEach = function (fn) {
        for (var i = 0; i < this.size; i++) {
            var index = new ObjectInt_1.default(i);
            fn.call(this.item(index), index, this);
        }
        return new ObjectVoid_1.default();
    };
    /**
     *
     * @param {ObjectInt} position
     * @returns {(Z | ObjectNull)}
     * @memberof ObjectArray
     */
    ObjectArray.prototype.item = function (position) {
        var value = this[position.value()];
        if (value === undefined) {
            return new ObjectNull_1.default();
        }
        return value;
    };
    /**
     * @returns {ObjectInt}
     * @memberof ObjectArray
     */
    ObjectArray.prototype.length = function () {
        return new ObjectInt_1.default(this.size);
    };
    /**
     * @returns {string}
     * @memberof ObjectArray
     */
    ObjectArray.prototype.name = function () {
        return "[object Array]";
    };
    /**
     * @returns {boolean}
     * @memberof ObjectArray
     */
    ObjectArray.prototype.nullable = function () {
        return false;
    };
    /**
     * @returns {Z}
     * @memberof ObjectArray
     */
    ObjectArray.prototype.pop = function () {
        var el = this[this.size - 1];
        delete this[this.size - 1];
        this.size = this.size - 1;
        return el;
    };
    /**
     * @param {Z} el
     * @returns {ObjectInt}
     * @memberof ObjectArray
     */
    ObjectArray.prototype.push = function (el) {
        var casted = (new this.matcher(ObjectArray.skipToken)).cast(el);
        this.size = this.size + 1;
        this[this.size] = casted;
        return this.length();
    };
    /**
     * @param {ObjectArray<Z>} [value]
     * @returns {typeof value extends undefined ? Z[] : []}
     * @memberof ObjectArray
     */
    ObjectArray.prototype.value = function (value) {
        if (value) {
            this.empty();
            this.size = value.length().value();
            for (var i = 0; i < this.size; i++) {
                this[i] = value[i];
            }
            return [];
        }
        var arr = [];
        for (var i = 0; i < this.length().value(); i++) {
            arr.push(this[i]);
        }
        return arr;
    };
    return ObjectArray;
}(TypeBase_1.default));
exports.default = ObjectArray;
