import { ConstructorOf } from '@Types/ConstructorOf';
export default abstract class TypeBase<TUnderlyingType = unknown> {
    /**
     * @private
     * @type {TUnderlyingType}
     * @memberof TypeBase
     */
    private underlyingValue;
    /**
     * @static
     * @memberof TypeBase
     */
    static skipToken: string;
    /**
     * Creates an instance of TypeBase.
     * @param {TUnderlyingType} value
     * @memberof TypeBase
     */
    constructor(value: TUnderlyingType);
    /**
     * @protected
     * @param {unknown} arg
     * @returns {string}
     * @memberof TypeBase
     */
    protected getUnderlyingTypeOf(arg: unknown): string;
    /**
     * @abstract
     * @param {unknown} type
     * @returns {TUnderlyingType}
     * @memberof TypeBase
     */
    abstract cast<T extends TypeBase>(type: T): TypeBase<TUnderlyingType>;
    /**
     * @param {TypeBase} type
     * @returns {boolean}
     * @memberof TypeBase
     */
    equalsTo<T extends new (...args: any[]) => TypeBase>(type: T): boolean;
    /**
     * @abstract
     * @returns {ConstructorOf<this>}
     * @memberof TypeBase
     */
    getType(): ConstructorOf<TypeBase>;
    /**
     * @memberof TypeBase
     */
    abstract name(): string;
    /**
     * @abstract
     * @returns {boolean}
     * @memberof TypeBase
     */
    abstract nullable(): boolean;
    /**
     * @returns {string}
     * @memberof TypeBase
     */
    toString(): string;
    /**
     * @abstract
     * @returns {TUnderlyingType}
     * @memberof TypeBase
     */
    value(value?: TypeBase<unknown>): TUnderlyingType;
}
