"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var InvalidCastError_1 = __importDefault(require("@InternalErrors/InvalidCastError"));
var TypeBase = /** @class */ (function () {
    /**
     * Creates an instance of TypeBase.
     * @param {TUnderlyingType} value
     * @memberof TypeBase
     */
    function TypeBase(value) {
        var uncastedType = Object.prototype.toString.call(value);
        var ownedType = this.name();
        if (!this.nullable() && (value === undefined || value === null)) {
            throw new InvalidCastError_1.default("Undefined argument in " + this.name());
        }
        else if (value !== TypeBase.skipToken && uncastedType === ownedType) {
            this.underlyingValue = value;
        }
        else if (value !== TypeBase.skipToken) {
            throw new InvalidCastError_1.default("Cannot convert " + uncastedType + " to " + ownedType);
        }
    }
    /**
     * @protected
     * @param {unknown} arg
     * @returns {string}
     * @memberof TypeBase
     */
    TypeBase.prototype.getUnderlyingTypeOf = function (arg) {
        return Object.prototype.toString.call(arg);
    };
    /**
     * @param {TypeBase} type
     * @returns {boolean}
     * @memberof TypeBase
     */
    TypeBase.prototype.equalsTo = function (type) {
        return new type(TypeBase.skipToken).name() === this.name();
    };
    /**
     * @abstract
     * @returns {ConstructorOf<this>}
     * @memberof TypeBase
     */
    TypeBase.prototype.getType = function () {
        return this.constructor.prototype.constructor;
    };
    ;
    /**
     * @returns {string}
     * @memberof TypeBase
     */
    TypeBase.prototype.toString = function () {
        return String(this.underlyingValue);
    };
    /**
     * @abstract
     * @returns {TUnderlyingType}
     * @memberof TypeBase
     */
    TypeBase.prototype.value = function (value) {
        if (!!value) {
            this.underlyingValue = this.cast(value).value();
        }
        return this.underlyingValue;
    };
    ;
    /**
     * @static
     * @memberof TypeBase
     */
    TypeBase.skipToken = '---invalidate';
    return TypeBase;
}());
exports.default = TypeBase;
